%COMPUTESCALEDEIGENVECTORS Compute weighted median isotropic BRDF,
%  mean mapped isotropic BRDF, weighted principal components
%  and mapped isotropic BRDFs with mean subtracted
%
%  [F_MED_W, MI, Q, Y] = COMPUTESCALEDEIGENVECTORS(A, W, MASKMAP)
%  computes median isotropic BRDF F_MED_W weighted by weights W,
%  mean mapped isotropic BRDF MI, weighted principal components Q
%  and mapped isotropic BRDFs Y with mean MI subtracted.
%  Parameters are original BRDF data A, cosine weights W and masking
%  map MASKMAP.
%
%  [F_MED_W, MI, Q, Y] = COMPUTESCALEDEIGENVECTORS(A, W, MASKMAP, E, K)
%  enables choosing value of parameter E (0.001) and number of returned
%  principal components K (all).
%  Default values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [f_med_w, mi, Q, Y] = computeScaledEigenvectors(A, w, MaskMap, e, k)

if (~exist('e','var') || isempty(e))
    e = 0.001;
end

fprintf('Computing scaled eigenvectors... ');

A = A(MaskMap,:);
if (~exist('k','var') || isempty(k))
    k = min(size(A));
end

m = size(A,2);

f_med_w = median(A,2).*w;

X = log((A.*repmat(w,1,m) + e)./repmat(f_med_w+e,1,m));
clear A;

mi = mean(X,2);

Y = X-repmat(mi,1,m);
clear X;

if (size(Y,2) < size(Y,1))
    [U, ~] = svd(Y'*Y);
    Q = Y*U(:,1:k);
else
    [U, S] = svd(Y*Y');
    Q = U(:,1:k)*sqrt(S(1:k,1:k));
end

fprintf('done.\n');
