%CONDITIONNUMBER Compute condition number when adding new column in paralell
%
%  C = CONDITIONNUMBER(A, U) computes condition number C when updating
%  matrix Q' where A = Q'*Q by adding new column U. Function computes
%  solution for many columns of U in paralell.
%
%  C = CONDITIONNUMBER(A, U, K) enables to specify number of iterations
%  K (30). Default values in brackets.
%
%  Function can be inaccurate due to inaccurate implementation of eig().
%  For more information see appendix B of
%
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function c = conditionNumber(A, U, k)

    if (~exist('k','var') || isempty(k))
        k = 30;
    end

    n = size(A,1);
    m = size(U,2);

    s = sum(U.^2,1);
    U = U./repmat(sqrt(s),n,1);

    [Q,D] = eig(A);
    d = diag(D);
    
    z2 = repmat((Q\U).^2,[1,1,2]);
    
    dl = repmat((repmat(d,[1,1,2])-repmat(shiftdim([d(1),d(n)],-1),n,1)),1,m)./repmat(s,[n,1,2]);
    tl = repmat(realmin,[1,m,2]);
    tr = permute([dl(2,:,1); ones(1,m)],[3,2,1]);
    t = find_t(tl, tr, z2, dl, n, k);
    L = repmat(shiftdim([d(1),d(n)],-1),1,m)+repmat(s,[1,1,2]).*t;
    L = abs(sqrt(L));
    
    c = L(:,:,2)./L(:,:,1);    
end

function t = find_t(tl, tr, z2, dl, n, k)
    for i = 1:k
        t = (tl+tr)/2;
        w = 1 + sum(z2./(dl-repmat(t,n,1)),1);
        tl(w<0) = t(w<0);
        tr(w>0) = t(w>0);
    end
    t = (tl+tr)/2;    
end
