%EXAMPLERECONSTRUCTION Simulate reconstruction of one material based on others
%
%  BRDF = EXAMPLERECONSTRUCTION(N, MATNAME) simulates reconstruction of
%  material specified by MATNAME using N directions. Reconstructed material
%  is excluded from the database and reconstruction is based on remaining
%  materials.
%
%  BRDF = EXAMPLERECONSTRUCTION(N, MATNAME, DBPATH, TMPPATH) enables to
%  specify directory with materials DBPATH ('./data') and temporary
%  directory TMPPATH ('./data1').
%  Default values in brackets.
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function BRDF = exampleReconstruction(n, matName, DBpath, TMPpath)

if (~exist('DBpath','var') || isempty(DBpath))
    DBpath = './data';
end
if (~exist('TMPpath','var') || isempty(TMPpath))
    TMPpath = './data1';
end

% move one material
if (~exist(TMPpath,'dir'))
    mkdir(TMPpath);
    cleanDir = 1;
end
movefile(fullfile(DBpath, matName), fullfile(TMPpath, matName));

% training
A149 = readBRDFdata(DBpath);
[w149, MaskMap149] = computeMaps(A149);
[f_med_w149, mi149, Q149] = computeScaledEigenvectors(A149, w149, MaskMap149);

% simulated measurement
A1 = readBRDFdata(TMPpath);
load(sprintf('directions/directions%.4d.mat', n), 'directions');
values = A1(mapAngleToIdx(directions),:);

% reconstruction
A1r = reconstruct(w149, MaskMap149, f_med_w149, mi149, Q149, directions, values);
BRDF = saveBRDFdata(A1r);

% clean up
movefile(fullfile(TMPpath, matName), fullfile(DBpath, matName));
if (exist('cleanDir','var') && ~isempty(cleanDir) && cleanDir == 1)
    rmdir(TMPpath);
    clear('cleanDir');
end
