%EXAMPLETRAINING Read BRDF data and compute everything needed
%  for optimization and reconstruction
%
%  [W, MASKMAP, MASKMAPLIMITED, F_MED_W, MI, Q, Y, HDANGLES]
%  = EXAMPLETRAINING() computes everything needed for optimization
%  and reconstruction. Weights W (cosine map) favors lower
%  elevations, mask map MASKMAP masks unmeasurable directions
%  and limited mask map MASKMAPLIMITED masks elevations over
%  75 degrees in addition. F_MED_W is median isotropic BRDF
%  weighted by weights W, MI is mean mapped isotropic BRDF,
%  Q are weighted principal components, Y are mapped isotropic
%  BRDFs with mean MI subtracted and HDANGLES is list of all
%  angles in half-diff parameterization.
%
%  [W, MASKMAP, MASKMAPLIMITED, F_MED_W, MI, Q, Y, HDANGLES]
%  = EXAMPLETRAINING(DBPATH, E, K, STEPS, BOUNDS, HDS)
%  enables to specify data directory DBPATH ('./data'),
%  parameter epsilon E (0.001) and number of returned principal
%  components K (all). BRDF data should be in standard
%  parameterization with steps between consecutive elevations
%  and azimuths specified by STEPS ([15 7.5 15 7.5])
%  in order theta_i, phi_i, theta_v, phi_v
%  and their maximal values specified by BOUNDS ([75 360 75 360]).
%  Data are converted into half-diff parameterization
%  with step specified by HDS (5).
%  Default values in brackets.
%
%  BRDF data can be downloaded in Matlab format from
%  UTIA BRDF Database at http://btf.utia.cas.cz/.
%  Please extract data into directory specified by DBpath
%  (default value './data').
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [w, MaskMap, MaskMapLimited, f_med_w, mi, Q, Y, hdAngles] = exampleTraining(DBpath, e, k, steps, bounds, hds)

if (~exist('DBpath','var') || isempty(DBpath))
    DBpath = [];
end
if (~exist('e','var') || isempty(e))
    e = [];
end
if (~exist('k','var') || isempty(k))
    k = [];
end
if (~exist('steps','var') || isempty(steps))
    steps = [];
end
if (~exist('bounds','var') || isempty(bounds))
    bounds = [];
end
if (~exist('hds','var') || isempty(hds))
    hds = [];
end

A = readBRDFdata(DBpath, steps, bounds, hds);

[w, MaskMap, MaskMapLimited, hdAngles] = computeMaps(A, e, hds);

[f_med_w, mi, Q, Y] = computeScaledEigenvectors(A, w, MaskMap, e, k);
