%MAPANGLETOIDX Map angle in half-diff parameterization to row (and valid
%row) of data
%
%  IDX = MAPANGLETOIDX(ANGLE) maps ANGLE in half-diff parameterization to
%  index IDX of row of data.
%
%  [IDX, IDXVALID] = MAPANGLETOIDX(ANGLE, MASKMAP) maps ANGLE in half-diff
%  parameterization to index IDX of row of data and to valid index IDXVALID
%  of valid row of data. MASKMAP selects valid rows of data.
%
%  MAPANGLETOIDX(..., HDS) enables to specify step in half-diff
%  parameterization HDS (5).
%  Default values in brackets.
%
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [idx, idxValid] = mapAngleToIdx(angle, MaskMap, hds)

if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

ntd = floor(89/hds)+1;
nth = floor(89/hds)+1;
npd = floor(179/hds)+1;

idx_pd = round(angle(:,1)/hds);
idx_pd(idx_pd<0) = 0;
idx_pd(idx_pd>=npd) = npd-1;

idx_td = round(angle(:,2)/hds);
idx_td(idx_td<0) = 0;
idx_td(idx_td>=ntd) = ntd-1;

idx_th = round(angle(:,3)/hds);
idx_th(idx_th<0) = 0;
idx_th(idx_th>=nth) = nth-1;

idx = idx_th*npd*ntd + idx_td*npd + idx_pd + 1;

if (exist('MaskMap','var') && ~isempty(MaskMap))
    idxValid = mapIdxToValid(idx, MaskMap);
else
    idxValid = [];
end
