%MAPIDXTOANGLE Map (valid) row of data to angle in half-diff
%parameterization
%
%  ANGLE = MAPIDXTOANGLE(IDX) maps index IDX of row of data to ANGLE in
%  half-diff parameterization.
%
%  ANGLE = MAPIDXTOANGLE(IDX, MASKMAP) maps valid index IDX of row of data
%  to ANGLE in half-diff parameterization. MASKMAP selects valid rows of
%  data.
%
%  ANGLE = MAPIDXTOANGLE(..., HDS) enables to specify step in half-diff
%  parameterization HDS (5).
%  Default values in brackets.
%
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function angle = mapIdxToAngle(idx, MaskMap, hds)

if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

ntd = floor(89/hds)+1;
npd = floor(179/hds)+1;

if (exist('MaskMap','var') && ~isempty(MaskMap))
    idx = mapValidToIdx(idx, MaskMap);
end

idx_th = floor((idx-1)/(npd*ntd));
idx_td = floor((idx-1-idx_th*npd*ntd)/npd);
idx_pd = idx-1-idx_th*npd*ntd-idx_td*npd;

angle = [idx_pd*hds idx_td*hds idx_th*hds];
