%OBJFUN Objective function
%
%  [E, GRADE] = OBJFUN(C, Q, Y, P, M, ETA, LAMBDA) computes value and
%  gradient of criterial function. See Equation 5 and Appendix A:
%  
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function [E, gradE] = objFun(C, Q, Y, P, M, eta, lambda)

A = Y-Q*C;
B = P*C*M;
E = sum(A(:).^2) + eta*sum(C(:).^2) + lambda*sum(B(:).^2);

if (nargout > 1)
    gradE = - 2*Q'*Y + 2*(Q'*Q)*C + 2*eta*C + 2*lambda*(P'*P)*C*(M*M);
end
