%ROWTO3D Compute indices of half-diff angles based on valid row
%
%  P = ROWTO3D(R, MASKMAP) computes indices P of half-diff angles based on
%  valid row R. MASKMAP selects valid rows of data.
%  
%  P = ROWTO3D(R, MASKMAP, HDS) computes indices P of half-diff angles
%  based on valid row R. MASKMAP selects valid rows of data. HDS (5)
%  specifies step in half-diff parameterization.
%  Default values in brackets.
%
%  Radomir Vavra and Jiri Filip: Minimal Sampling
%  for Effective Acquisition of Anisotropic BRDFs.
%  Pacific Graphics 2016.
%
function p = rowTo3D(r, MaskMap, hds)

if (~exist('hds','var') || isempty(hds))
    hds = 5;
end

ntd = floor(89/hds)+1;
npd = floor(179/hds)+1;

idx = mapValidToIdx(r, MaskMap);

idx_th = floor((idx-1)/(npd*ntd));
idx_td = floor((idx-1-idx_th*npd*ntd)/npd);
idx_pd = idx-1-idx_th*npd*ntd-idx_td*npd;

p = [idx_pd, idx_td, idx_th];
